import logging
from pathlib import Path
from rdflib import Graph, RDF

from .rdf_helper import SCOREP, SCOREP_JSONLD_FILE


def add_arguments_get_id(subparsers):
    parser_add = subparsers.add_parser("get-id", help="Add a Scorep experiment")
    parser_add.add_argument(
        "experiment_path",
        type=lambda p: Path(p).resolve(),
        help="Path to Scorep experiment",
    )
    parser_add.set_defaults(func=get_id_function)


def handle_get_id(experiment_path, metadata_file_name) -> bool:
    new_graph = Graph().parse(
        str(experiment_path / metadata_file_name), format="json-ld"
    )

    subject = list(new_graph.subjects(predicate=RDF.type, object=SCOREP.Experiment))
    assert len(subject) == 1
    subject = subject[0]

    print(str(subject))


def get_id_function(args):
    metadata_file_name = SCOREP_JSONLD_FILE

    experiment_path: Path = args.experiment_path

    if not experiment_path.exists():
        logging.error(f"Experiment path '{experiment_path}' does not exist. Aborting.")
        return

    metadata_file = experiment_path / metadata_file_name
    if not metadata_file.exists():
        logging.error(f"Metadata file '{metadata_file}' does not exist. Aborting.")
        return
    elif metadata_file.stat().st_size == 0:
        logging.error(f"Metadata file '{metadata_file}' is empty. Aborting.")
        return

    handle_get_id(experiment_path, metadata_file_name)



# scorep_db/object_store.py

from abc import ABC, abstractmethod
import logging
import os
import uuid
from pathlib import Path
from dotenv import load_dotenv
from minio import Minio
from minio.error import S3Error
import shutil


class ObjectStore(ABC):
    def __init__(self, config_file: Path):
        self.config_file = config_file
        load_dotenv(str(self.config_file))

    @abstractmethod
    def upload_experiment(
        self, experiment_path: Path, new_experiment_directory_path: str
    ) -> None:
        pass

    @abstractmethod
    def download_experiment(self, source_path: str, target_path: Path) -> None:
        pass

    @abstractmethod
    def generate_new_experiment_path(self) -> str:
        pass

    @abstractmethod
    def clear_storage(self) -> None:
        pass

    @abstractmethod
    def get_storage_path(self) -> str:
        pass

    @abstractmethod
    def health_check(self) -> bool:
        """Check if the storage is reachable and configured correctly."""
        pass


class OfflineObjectStore(ObjectStore):
    def __init__(self, config_file: Path):
        super().__init__(config_file)
        self.offline_directory: Path = Path(
            os.path.expandvars(os.getenv("SCOREP_DB_OFFLINE_DIRECTORY"))
        )

    @staticmethod
    def _copy_recursive(source, target):
        src_dir = source
        dst_dir = target
        os.makedirs(dst_dir, exist_ok=True)

        for item in os.listdir(src_dir):
            s = os.path.join(src_dir, item)
            d = os.path.join(dst_dir, item)
            if os.path.isdir(s):
                shutil.copytree(s, d, dirs_exist_ok=True)
            else:
                shutil.copy2(s, d)

        logging.info(f"Copied experiment directory to {dst_dir}")

    def upload_experiment(
        self, experiment_path: Path, new_experiment_directory_path: str
    ) -> None:
        target = self.offline_directory / Path(new_experiment_directory_path)
        self._copy_recursive(experiment_path, target)

    def download_experiment(self, source: Path, target: Path) -> None:
        source_ = self.offline_directory / source
        self._copy_recursive(source_, target)

    def generate_new_experiment_path(self) -> str:
        return str(uuid.uuid4())

    def clear_storage(self) -> None:
        directory_root = self.offline_directory
        if directory_root.exists():
            for item in directory_root.iterdir():
                if item.is_dir():
                    shutil.rmtree(item)
                else:
                    item.unlink()
            logging.info(
                f"Cleared all contents within the offline storage at {directory_root}"
            )
        else:
            logging.info(f"No offline storage found at {directory_root}")

    def get_storage_path(self) -> str:
        return str(self.offline_directory)

    def health_check(self):
        """Check if the offline directory exists and is writable."""
        if self.offline_directory.exists() and os.access(
            self.offline_directory, os.W_OK
        ):
            logging.info(
                f"Offline storage directory '{self.offline_directory}' is accessible and writable."
            )
        else:
            raise Exception(
                f"Offline storage directory '{self.offline_directory}' is not accessible or writable."
            )


class OnlineObjectStore(ObjectStore):
    def __init__(self, config_file: Path):
        super().__init__(config_file)
        self.endpoint = f"{os.getenv('SCOREP_DB_ONLINE_OBJ_HOSTNAME')}:{os.getenv('SCOREP_DB_ONLINE_OBJ_PORT')}"
        self.access_key = os.getenv("SCOREP_DB_ONLINE_OBJ_USER")
        self.secret_key = os.getenv("SCOREP_DB_ONLINE_OBJ_PASSWORD")

        self.bucket_name = os.getenv("SCOREP_DB_ONLINE_OBJ_BUCKET_NAME")

        self.client = Minio(
            endpoint=self.endpoint,
            access_key=self.access_key,
            secret_key=self.secret_key,
            secure=False,
        )

    def upload_experiment(
        self, experiment_path: Path, new_experiment_directory_path: str
    ) -> None:
        self.ensure_bucket_exists(self.bucket_name)

        base_dir = new_experiment_directory_path

        for root, dirs, files in os.walk(experiment_path):
            for file_name in files:
                file_path = os.path.join(root, file_name)
                object_name = os.path.relpath(file_path, experiment_path)
                if base_dir:
                    object_name = os.path.join(base_dir, object_name)

                try:
                    self.client.fput_object(self.bucket_name, object_name, file_path)
                    logging.info(
                        f"Successfully uploaded {file_name} to {self.bucket_name}/{object_name}"
                    )
                except S3Error as e:
                    logging.error(f"Failed to upload {file_name}: {e}")

    def download_experiment(
        self, experiment_directory_path: str, local_directory: Path
    ) -> None:
        try:
            objects = self.client.list_objects(
                self.bucket_name, prefix=experiment_directory_path, recursive=True
            )
            for obj in objects:
                # Construct the local file path
                local_file_path = local_directory / obj.object_name[
                    len(experiment_directory_path) :
                ].lstrip("/")
                local_file_path.parent.mkdir(parents=True, exist_ok=True)

                # Download the object
                self.client.fget_object(
                    self.bucket_name, obj.object_name, str(local_file_path)
                )
                logging.info(
                    f"Successfully downloaded {obj.object_name} to {local_file_path}"
                )

        except S3Error as e:
            logging.error(
                f"Failed to download experiment from {experiment_directory_path}: {e}"
            )

    def ensure_bucket_exists(self, bucket_name):
        if not self.client.bucket_exists(bucket_name):
            self.client.make_bucket(bucket_name)
            logging.info(f"Bucket '{bucket_name}' created.")
        else:
            logging.info(f"Bucket '{bucket_name}' already exists.")

    def generate_new_experiment_path(self) -> str:
        return str(uuid.uuid4())

    def clear_storage(self) -> None:
        try:
            objects = self.client.list_objects(self.bucket_name, recursive=True)
            for obj in objects:
                self.client.remove_object(self.bucket_name, obj.object_name)
            logging.info(f"Cleared online storage bucket '{self.bucket_name}'")
        except S3Error as e:
            logging.error(f"Failed to clear online storage: {e}")

    def get_storage_path(self) -> str:
        return f"{self.endpoint}/{self.bucket_name}"

    def health_check(self):
        """Check if the online object store is reachable and bucket exists."""
        try:
            if self.client.bucket_exists(self.bucket_name):
                logging.info(
                    f"Online storage bucket '{self.bucket_name}' is reachable."
                )
            else:
                raise Exception(
                    f"Online storage bucket '{self.bucket_name}' does not exist."
                )

        except S3Error as e:
            raise Exception(
                f"Failed to check online storage bucket '{self.bucket_name}': {e}"
            )


def get_object_store(config_file: Path, mode: str) -> ObjectStore:
    if mode == "offline":
        return OfflineObjectStore(config_file)
    elif mode == "online":
        return OnlineObjectStore(config_file)
    else:
        logging.error(f"Unknown mode '{mode}'. Aborting.")
        raise ValueError(f"Unknown mode '{mode}'")



# scorep_db/rdf_database.py

from abc import ABC, abstractmethod
import logging
import os
from pathlib import Path
from dotenv import load_dotenv
from rdflib import Graph, URIRef
from rdflib_sqlalchemy import registerplugins
from rdflib_sqlalchemy.store import SQLAlchemy
import contextlib

registerplugins()


class RDFDatabase(ABC):
    def __init__(self, config_file: Path):
        self.config_file = config_file
        load_dotenv(str(self.config_file))

    @abstractmethod
    def get_graph(self) -> Graph:
        pass

    @staticmethod
    def open_graph(db_path: str) -> Graph:
        static_identifier = URIRef("http://internal.scorep.org/")
        with contextlib.suppress(TypeError):
            store = SQLAlchemy(identifier=None)
            graph = Graph(store=store, identifier=static_identifier)

        try:
            graph.open(db_path, create=False)
        except RuntimeError:
            graph.open(db_path, create=True)

        return graph

    @abstractmethod
    def get_database_path(self) -> str:
        pass

    @abstractmethod
    def health_check(self) -> bool:
        """Check if the RDF database is reachable and configured correctly."""
        pass


class OfflineRDFDatabase(RDFDatabase):
    def __init__(self, config_file: Path):
        super().__init__(config_file)
        self.scorep_db_path = Path(
            os.path.expandvars(os.getenv("SCOREP_DB_OFFLINE_PATH"))
        )
        self.scorep_db_name = Path(os.getenv("SCOREP_DB_OFFLINE_NAME"))

    def get_graph(self) -> Graph:
        db_path = f"sqlite:///{self.scorep_db_path / self.scorep_db_name}"

        logging.info(f"Connecting to offline database at {db_path}")

        return self.open_graph(db_path)

    def get_database_path(self) -> str:
        return str(self.scorep_db_path / self.scorep_db_name)

    def health_check(self):
        """Check if the offline RDF database file exists and is accessible."""
        if self.scorep_db_path.exists():  # and self.scorep_db_name.exists():
            logging.info(
                f"Offline RDF database file '{self.scorep_db_path / self.scorep_db_name}' is accessible."
            )
        else:
            raise Exception(
                f"Offline RDF database file '{self.scorep_db_path / self.scorep_db_name}' is not accessible."
            )


class OnlineRDFDatabase(RDFDatabase):
    def __init__(self, config_file: Path):
        super().__init__(config_file)
        self.hostname = os.getenv("SCOREP_DB_ONLINE_RDF_HOSTNAME")
        self.port = os.getenv("SCOREP_DB_ONLINE_RDF_PORT")
        self.user = os.getenv("SCOREP_DB_ONLINE_RDF_USER")
        self.password = os.getenv("SCOREP_DB_ONLINE_RDF_PASSWORD")
        self.scorep_db_name = os.getenv("SCOREP_DB_ONLINE_RDF_DB_NAME")

    def get_graph(self) -> Graph:
        db_path = self._get_db_path()

        logging.info(f"Connecting to online database at {db_path}")

        return self.open_graph(db_path)

    def get_database_path(self) -> str:
        return f"{self.user}@{self.hostname}:{self.port}/{self.scorep_db_name}"

    def health_check(self) -> bool:
        """Check if the online RDF database is reachable by attempting a connection."""
        try:
            db_path = self._get_db_path()
            graph = self.open_graph(db_path)
            if graph:
                logging.info(
                    f"Successfully connected to the online RDF database at '{db_path}'."
                )

            else:
                raise Exception(
                    f"Failed to connect to the online RDF database at '{db_path}'."
                )
        except Exception as e:
            raise Exception(f"Failed to check online RDF database: '{e}'")

    def _get_db_path(self) -> str:
        return f"postgresql+psycopg2://{self.user}:{self.password}@{self.hostname}:{self.port}/{self.scorep_db_name}"


def get_rdf_database(config_file: Path, mode: str) -> RDFDatabase:
    if mode == "offline":
        return OfflineRDFDatabase(config_file)
    elif mode == "online":
        return OnlineRDFDatabase(config_file)
    else:
        logging.error(f"Unknown mode '{mode}'. Aborting.")
        raise ValueError(f"Unknown mode '{mode}'")



import logging
from pathlib import Path
from rdflib import Graph
from .rdf_database import get_rdf_database
from .object_store import get_object_store
from .exceptions import IllFormedQueryError


def add_arguments_download(subparsers):
    parser_query = subparsers.add_parser(
        "download",
        help="Download the experiments that are the result of a SPARQL query",
    )
    parser_query.add_argument(
        "config_file", type=lambda p: Path(p).resolve(), help="Path to config file"
    )
    parser_query.add_argument(
        "mode", choices=["offline", "online"], help="Operation mode"
    )
    parser_query.add_argument(
        "query_file", type=lambda p: Path(p).resolve(), help="Path to SPARQL query file"
    )
    parser_query.add_argument(
        "download_directory",
        type=lambda p: Path(p).resolve(),
        help="Path to the downloaded files",
    )

    parser_query.add_argument(
        "--dryrun",
        action="store_true",
        help="Simulate the download process without downloading files",
    )

    parser_query.set_defaults(func=download_function)


def handle_download(mode, config_file, sparql_query, download_directory, dryrun=False):
    rdf_database: Graph = get_rdf_database(config_file, mode).get_graph()

    # Execute the query
    try:
        results = rdf_database.query(sparql_query)

        logging.info("Query executed successfully.")

    except Exception as e:
        logging.error(f"Failed to execute query: {e}")

    # Handle the results.

    already_downloaded = {}

    for i, row in enumerate(results, start=1):
        # Iterate Result Subjects
        experiment_id: str = None
        store_path: str = None

        new_name = {}
        for var in row.labels:
            key = str(var).strip()
            value = str(row[var]).strip()

            if value == "":
                # This might not be needed? This prevents from empty keys being present
                continue

            if key.lower() == "experiment":
                experiment_id = value
                continue

            if key.lower() == "storepath":
                store_path = value
                continue

            # Collect key value pairs for the download
            new_name[str(key)] = str(value.strip())

        if experiment_id is None:
            raise IllFormedQueryError(
                f"The used SPARQL query has no '?Experiment' result, but this is needed!"
            )
        if store_path is None:
            raise IllFormedQueryError(
                f"The used SPARQL query has no '?StorePath' result, but this is needed!"
            )

        print(f"Working on '{experiment_id}'")

        if not new_name:
            # Use "old" store path as fallback
            new_name[store_path] = ""

        row_result = {"source_path": store_path, "target_path": new_name}

        if already_downloaded.get(experiment_id, False):
            logging.info(f"Updating exsiting entry for {experiment_id}")
            # There was already a matching experiment
            #  --> Update target path with new path
            existing_experiment = already_downloaded.get(experiment_id)

            existing_experiment["target_path"].update(new_name)

        else:
            logging.info(f"New entry for {experiment_id}")
            # No previous entry
            already_downloaded[experiment_id] = row_result
        print("-" * 40)

    if dryrun:
        logging.info("Dry run mode enabled. The following files would be downloaded:")
    else:
        object_store = get_object_store(config_file, mode)

    for id, v in already_downloaded.items():
        source_path = v["source_path"]

        new_target_name = v["target_path"]

        new_name_string = []
        for sorted_keys in sorted(new_target_name.keys()):
            new_name_string.append(sorted_keys + "_" + new_target_name[sorted_keys])

        merged_string = ".".join(new_name_string)

        # Append the run_id to the folder to make sure that no overwriting takes place.
        merged_string += f".{id.split('/')[-1]}"

        target_path: Path = Path(download_directory) / Path(merged_string)

        if dryrun:
            logging.info(
                f"Would download\n\t'{id}'\nfrom\n\t'{source_path}'\nto\n\t'{target_path}'"
            )
        else:
            object_store.download_experiment(source_path, target_path)
            logging.info(f"Downloaded\t\n'{id}'\nto\n\t'{target_path}'")

    rdf_database.close()


def download_function(args):
    config_file: Path = args.config_file
    mode: str = args.mode
    query_file: Path = args.query_file
    download_directory: Path = args.download_directory
    dryrun: bool = args.dryrun

    if not config_file.exists():
        logging.error(f"Config file '{config_file}' does not exist. Aborting.")
        return

    if not query_file.exists():
        logging.error(f"Query file '{query_file}' does not exist. Aborting.")
        return

    if not download_directory.exists():
        logging.error(
            f"Download destination directory '{download_directory}' does not exist. Aborting."
        )
        return

    # Read the query
    with query_file.open("r") as file:
        sparql_query = file.read()

    logging.info(f"Executing SPARQL query from file: {query_file}")

    handle_download(mode, config_file, sparql_query, download_directory, dryrun)
    # Connect to the RDF database



import logging
from pathlib import Path
from .rdf_database import get_rdf_database


def add_arguments_query(subparsers):
    parser_query = subparsers.add_parser(
        "query", help="Execute a SPARQL query on the RDF database"
    )
    parser_query.add_argument(
        "config_file", type=lambda p: Path(p).resolve(), help="Path to config file"
    )
    parser_query.add_argument(
        "mode", choices=["offline", "online"], help="Operation mode"
    )
    parser_query.add_argument(
        "query_file", type=lambda p: Path(p).resolve(), help="Path to SPARQL query file"
    )
    parser_query.set_defaults(func=query_function)


def query_function(args):
    config_file: Path = args.config_file
    mode: str = args.mode
    query_file: Path = args.query_file

    if not config_file.exists():
        logging.error(f"Config file '{config_file}' does not exist. Aborting.")
        return

    if not query_file.exists():
        logging.error(f"Query file '{query_file}' does not exist. Aborting.")
        return

    # Read the query
    with query_file.open("r") as file:
        sparql_query = file.read()

    logging.info(f"Executing SPARQL query from file: {query_file}")

    # Connect to the RDF database
    rdf_database = get_rdf_database(config_file, mode).get_graph()

    # Execute the query
    try:
        results = rdf_database.query(sparql_query)

        for i, row in enumerate(results, start=1):
            print(f"Result {i}:")
            for var in row.labels:
                print(f"  {var}: {row[var]}")
            print("-" * 40)

        logging.info("Query executed successfully.")

    except Exception as e:
        logging.error(f"Failed to execute query: {e}")

    finally:
        rdf_database.close()



from pathlib import Path
import logging
from rdflib import Graph, Namespace, Literal, RDF

from .rdf_database import get_rdf_database
from .object_store import get_object_store

from .rdf_helper import SCOREP, SCOREP_JSONLD_FILE, test_if_already_exists


def add_arguments_merge(subparsers):
    parser_add = subparsers.add_parser(
        "merge", help="Merge an offline database into an online database"
    )
    parser_add.add_argument(
        "config_file", type=lambda p: Path(p).resolve(), help="Path to config file"
    )
    parser_add.add_argument(
        "--dryrun",
        action="store_true",
        help="Simulate the download process without downloading files",
    )

    parser_add.set_defaults(func=merge_function)


def handle_merge(config_file: Path, dryrun: bool = False):
    local_rdf_db: Graph = get_rdf_database(config_file, "offline").get_graph()
    remote_rdf_db: Graph = get_rdf_database(config_file, "online").get_graph()

    local_object_store = get_object_store(config_file, "offline")
    remote_object_store = get_object_store(config_file, "online")

    local_experiments = list(
        local_rdf_db.subjects(predicate=RDF.type, object=SCOREP.Experiment)
    )
    assert len(local_experiments) > 1

    for local_experiment in local_experiments:
        if (local_experiment, RDF.type, SCOREP.Experiment) in remote_rdf_db:
            continue
        if not local_experiment:
            print("None local experiment")

            continue

        store_path = None
        for obj in local_rdf_db.objects(
            subject=local_experiment, predicate=SCOREP.storePath
        ):
            store_path = obj
            break
        print(f"{store_path=}")
        if not store_path:
            logging.error(
                f"Entry '{local_experiment}' has no store path! This should not have happened."
            )
            raise ValueError

        local_store_path = Path(str(store_path))

        local_db_root = Path(local_object_store.get_storage_path())
        # Reuse the old name for upload.
        # Directly upload the files without moving them into a local storage

        if dryrun:
            logging.info(f"Dryrun: Would upload {local_experiment}.")
        else:
            logging.info(f"Upload {local_experiment}.")
            remote_object_store.upload_experiment(
                local_db_root / local_store_path, local_store_path
            )

    if dryrun:
        logging.info(f"Dryrun: Would merge graphs.")
    else:
        logging.info(f"Merging graphs.")
        remote_rdf_db += local_rdf_db
        remote_rdf_db.commit()

    remote_rdf_db.close()
    local_rdf_db.close()

    return


def merge_function(args):
    config_file: Path = args.config_file
    dryrun: bool = args.dryrun

    if not config_file.exists():
        logging.error(f"File '{config_file}' does not exist. Abort.")

    handle_merge(config_file, dryrun)



from rdflib import Namespace, Graph, RDF, URIRef
import logging
from contextlib import contextmanager

SCOREP = Namespace("http://scorep-fair.github.io/ontology#")
SCOREP_JSONLD_FILE = "scorep.fair.json"


def test_if_already_exists(database: Graph, new_entry: Graph, key: URIRef) -> bool:
    subject = list(new_entry.subjects(predicate=RDF.type, object=key))
    assert len(subject) == 1
    subject = subject[0]

    if (subject, RDF.type, SCOREP.Experiment) in database:
        logging.info(f"The subject '{subject}' already exists in the existing graph.")
        return True
    else:
        logging.info(f"The subject '{subject}' does not exist in the existing graph.")
        return False



from .rdf_database import get_rdf_database
from .object_store import get_object_store

from pathlib import Path
import logging
import sys


def add_arguments_health_check(subparsers):
    parser_health_check = subparsers.add_parser(
        "health_check", help="Verify database accessibility"
    )
    parser_health_check.add_argument(
        "config_file", type=lambda p: Path(p).resolve(), help="Path to config file"
    )
    parser_health_check.add_argument(
        "mode", choices=["offline", "online"], help="Operation mode"
    )
    parser_health_check.set_defaults(func=health_check_function)


def handle_health_check(config_file: Path, mode: str):
    rdf_db = get_rdf_database(config_file, mode)
    rdf_db.health_check()

    object_store = get_object_store(config_file, mode)
    object_store.health_check()


def health_check_function(args):
    config_file: Path = args.config_file
    mode: str = args.mode

    if not config_file.exists():
        logging.error(f"File '{config_file}' does not exist. Abort.")
        return

    handle_health_check(config_file, mode)



# scorep_db/add.py

import logging
from pathlib import Path
from rdflib import Graph, RDF, Namespace, Literal

from .rdf_database import get_rdf_database
from .object_store import get_object_store

from .rdf_helper import SCOREP, SCOREP_JSONLD_FILE, test_if_already_exists


def add_arguments_add(subparsers):
    parser_add = subparsers.add_parser("add", help="Add a Scorep experiment")
    parser_add.add_argument(
        "config_file", type=lambda p: Path(p).resolve(), help="Path to config file"
    )
    parser_add.add_argument(
        "mode", choices=["online", "offline"], help="Operation mode"
    )
    parser_add.add_argument(
        "experiment_path",
        type=lambda p: Path(p).resolve(),
        help="Path to Scorep experiment",
    )
    parser_add.add_argument(
        "append_files",
        type=lambda p: Path(p).resolve(),
        nargs="*",
        help="Optional additional JSON-LD files to append and merge",
    )
    parser_add.set_defaults(func=add_function)


def handle_add(
    mode: str,
    config_file: Path,
    experiment_path: Path,
    metadata_file_name: str,
    append_files: list[Path] = None,
) -> None:
    rdf_database = get_rdf_database(config_file, mode)
    object_store = get_object_store(config_file, mode)

    existing_graph = rdf_database.get_graph()

    new_graph = Graph().parse(
        str(experiment_path / metadata_file_name), format="json-ld"
    )

    # Parse and merge additional JSON-LD files if provided
    if append_files:
        for file_path in append_files:
            logging.info(f"Appending JSON-LD file: {file_path}")
            additional_graph = Graph().parse(str(file_path), format="json-ld")
            new_graph += additional_graph

        turtle_data = new_graph.serialize(format="turtle")


    already_merged = test_if_already_exists(
        existing_graph, new_graph, SCOREP.Experiment
    )

    if already_merged:
        new_graph.close()
        existing_graph.close()
        return

    new_experiment_directory_path = object_store.generate_new_experiment_path()

    subject = list(new_graph.subjects(predicate=RDF.type, object=SCOREP.Experiment))[0]
    new_graph.add((subject, SCOREP.storePath, Literal(new_experiment_directory_path)))

    new_graph.commit()

    logging.info(f"Added new attribute to {subject}")

    object_store.upload_experiment(experiment_path, new_experiment_directory_path)

    existing_graph += new_graph
    new_graph.close()
    existing_graph.commit()
    existing_graph.close()


def add_function(args):
    metadata_file_name = SCOREP_JSONLD_FILE

    config_file: Path = args.config_file
    mode: str = args.mode
    experiment_path: Path = args.experiment_path

    append_files: list[Path] = args.append_files if args.append_files else []

    if not config_file.exists():
        logging.error(f"Config file '{config_file}' does not exist. Aborting.")
        return

    if not experiment_path.exists():
        logging.error(f"Experiment path '{experiment_path}' does not exist. Aborting.")
        return

    metadata_file = experiment_path / metadata_file_name
    if not metadata_file.exists():
        logging.error(f"Metadata file '{metadata_file}' does not exist. Aborting.")
        return
    elif metadata_file.stat().st_size == 0:
        logging.error(f"Metadata file '{metadata_file}' is empty. Aborting.")
        return

    logging.info(
        f"Adding experiment with config: {config_file}, mode: {mode}, path: {experiment_path}"
    )
    handle_add(mode, config_file, experiment_path, metadata_file_name, append_files)



# scorep_db/main.py

import argparse
import logging
from .command_add import add_arguments_add
from .command_merge import add_arguments_merge
from .command_clear import add_arguments_clear
from .command_query import add_arguments_query
from .command_get_id import add_arguments_get_id
from .command_download import add_arguments_download
from .command_health_check import add_arguments_health_check


def main():
    logging.basicConfig(
        level=logging.INFO,
        format="%(asctime)s - %(levelname)s - %(message)s",
        datefmt="%H:%M:%S",
    )
    parser = argparse.ArgumentParser(
        prog="scorep-db", description="Scorep-DB Command Line Tool"
    )
    subparsers = parser.add_subparsers(dest="command", required=True)

    add_arguments_add(subparsers)
    add_arguments_merge(subparsers)
    add_arguments_clear(subparsers)
    add_arguments_query(subparsers)
    add_arguments_get_id(subparsers)
    add_arguments_download(subparsers)
    add_arguments_health_check(subparsers)

    args = parser.parse_args()
    args.func(args)


if __name__ == "__main__":
    main()



class IllFormedQueryError(Exception):
    """IllFormedQueryError is raised if a query does not contain the needed results"""

    def __init__(self, message, code=""):
        self.message = message
        self.code = code
        super().__init__(self.message)

    def __str__(self):
        return f"IllFormedQueryError [Code {self.code}]: {self.message}"



import logging
from pathlib import Path
from .rdf_database import get_rdf_database
from .object_store import get_object_store


def add_arguments_clear(subparsers):
    parser_clear = subparsers.add_parser(
        "clear", help="Clear the databases and storage"
    )
    parser_clear.add_argument(
        "config_file", type=lambda p: Path(p).resolve(), help="Path to config file"
    )
    parser_clear.add_argument(
        "mode", choices=["online", "offline"], help="Operation mode"
    )
    parser_clear.set_defaults(func=clear_function)


def clear_function(args):
    config_file: Path = args.config_file
    mode: str = args.mode

    if not config_file.exists():
        logging.error(f"Config file '{config_file}' does not exist. Aborting.")
        return

    logging.info(f"Clearing databases and storage for mode: {mode}")

    rdf_database = get_rdf_database(config_file, mode)
    object_store = get_object_store(config_file, mode)

    confirmation = input(
        f"Are you sure you want to clear all data in the following paths?\n"
        f"RDF Database: '{rdf_database.get_database_path()}'\n"
        f"Object Store: '{object_store.get_storage_path()}'\n"
        f"This action cannot be undone. (yes/no): "
    )
    if confirmation.lower() != "yes":
        logging.info("Clear operation aborted by the user.")
        return

    logging.info("User confirmed. Proceeding with clearing operation.")

    # Clear RDF Database

    existing_graph = rdf_database.get_graph()
    existing_graph.remove((None, None, None))  # Remove all triples
    existing_graph.commit()
    existing_graph.close()
    logging.info("RDF database cleared.")

    # Clear Object Store

    object_store.clear_storage()
    logging.info("Object store cleared.")
